@@ -1,3 +1,11 @@
+local info = {
+    version   = 1.002,
+    comment   = "scintilla lpeg lexer for lua",
+    author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
+    copyright = "PRAGMA ADE / ConTeXt Development Team",
+    license   = "see context related readme files",
+}
+
 local lexer = lexer
 local token = lexer.token
 local P = lpeg.P
