@@ -11,15 +11,14 @@ local context   = context
 local generics  = context.generics -- needs documentation
 local variables = interfaces.variables
 
-generics.starttabulate = "start" .. variables.tabulate -- todo: e!start
-generics.stoptabulate  = "stop"  .. variables.tabulate -- todo: e!stop
+generics.starttabulate = "starttabulate" -- "start" .. variables.tabulate -- todo: e!start
+generics.stoptabulate  = "stoptabulate"  -- "stop"  .. variables.tabulate -- todo: e!stop
 
 local NC, NR = context.NC, context.NR
 
 local function tabulaterow(how,...)
-    local t = { ... }
-    for i=1,#t do
-        local ti = tostring(t[i])
+    for i=1,select("#",...) do
+        local ti = tostring(select(i,...))
         NC()
         if how then
             context[how](ti)
