@@ -6,13 +6,12 @@ if not modules then modules = { } end modules ['font-ext'] = {
     license   = "see context related readme files"
 }
 
-local utf = unicode.utf8
 local next, type, byte = next, type, string.byte
-local gmatch, concat = string.gmatch, table.concat
+local gmatch, concat, format = string.gmatch, table.concat, string.format
 local utfchar = utf.char
-local getparameters = utilities.parsers.getparameters
 
-local allocate = utilities.storage.allocate
+local commands, context = commands, context
+local fonts, utilities = fonts, utilities
 
 local trace_protrusion = false  trackers.register("fonts.protrusion", function(v) trace_protrusion = v end)
 local trace_expansion  = false  trackers.register("fonts.expansion",  function(v) trace_expansion  = v end)
@@ -20,19 +19,28 @@ local trace_expansion  = false  trackers.register("fonts.expansion",  function(v
 local report_expansions  = logs.reporter("fonts","expansions")
 local report_protrusions = logs.reporter("fonts","protrusions")
 
+-- todo: byte(..) => 0xHHHH
+
 --[[ldx--
 <p>When we implement functions that deal with features, most of them
 will depend of the font format. Here we define the few that are kind
 of neutral.</p>
 --ldx]]--
 
-local fonts              = fonts
+local handlers           = fonts.handlers
+local hashes             = fonts.hashes
+local otf                = handlers.otf
 
-local otffeatures        = fonts.constructors.newfeatures("otf")
-local registerotffeature = otffeatures.register
+local registerotffeature = handlers.otf.features.register
+local registerafmfeature = handlers.afm.features.register
 
-local afmfeatures        = fonts.constructors.newfeatures("afm")
-local registerafmfeature = afmfeatures.register
+local fontdata           = hashes.identifiers
+
+local allocate           = utilities.storage.allocate
+local settings_to_array  = utilities.parsers.settings_to_array
+local getparameters      = utilities.parsers.getparameters
+
+local setmetatableindex  = table.setmetatableindex
 
 -- -- -- -- -- --
 -- shared
@@ -57,14 +65,15 @@ end
 -- expansion (hz)
 -- -- -- -- -- --
 
-fonts.expansions   = allocate()
-local expansions   = fonts.expansions
+local expansions   = fonts.expansions or allocate()
 
-expansions.classes = allocate()
-local classes      = expansions.classes
+fonts.expansions   = expansions
 
-expansions.vectors = allocate()
-local vectors      = expansions.vectors
+local classes      = expansions.classes or allocate()
+local vectors      = expansions.vectors or allocate()
+
+expansions.classes = classes
+expansions.vectors = vectors
 
 -- beware, pdftex itself uses percentages * 10
 
@@ -102,7 +111,7 @@ local function initializeexpansion(tfmdata,value)
                 local step    = class.step    or 0
                 local factor  = class.factor  or 1
                 if trace_expansion then
-                    report_expansions("setting class %s, vector: %s, factor: %s, stretch: %s, shrink: %s, step: %s",
+                    report_expansions("setting class %a, vector %a, factor %a, stretch %a, shrink %a, step %a",
                         value,class.vector,factor,stretch,shrink,step)
                 end
                 tfmdata.parameters.expansion = {
@@ -135,10 +144,10 @@ local function initializeexpansion(tfmdata,value)
                     end
                 end
             elseif trace_expansion then
-                report_expansions("unknown vector '%s' in class '%s",class.vector,value)
+                report_expansions("unknown vector %a in class %a",class.vector,value)
             end
         elseif trace_expansion then
-            report_expansions("unknown class '%s'",value)
+            report_expansions("unknown class %a",value)
         end
     end
 end
@@ -340,14 +349,14 @@ local function map_opbd_onto_protrusion(tfmdata,value,opbd)
                 local data = lookuphash[lookup]
                 if data then
                     if trace_protrusion then
-                        report_protrusions("setting left using lfbd lookup '%s'",lookup)
+                        report_protrusions("setting left using lfbd lookup %a",lookup)
                     end
                     for k, v in next, data do
                     --  local p = - v[3] / descriptions[k].width-- or 1 ~= 0 too but the same
                         local p = - (v[1] / 1000) * factor * left
                         characters[k].left_protruding = p
                         if trace_protrusion then
-                            report_protrusions("lfbd -> %s -> U+%05X (%s) -> %0.03f (%s)",lookup,k,utfchar(k),p,concat(v," "))
+                            report_protrusions("lfbd -> %s -> %C -> %0.03f (% t)",lookup,k,p,v)
                         end
                     end
                     done = true
@@ -363,14 +372,14 @@ local function map_opbd_onto_protrusion(tfmdata,value,opbd)
                 local data = lookuphash[lookup]
                 if data then
                     if trace_protrusion then
-                        report_protrusions("setting right using rtbd lookup '%s'",lookup)
+                        report_protrusions("setting right using rtbd lookup %a",lookup)
                     end
                     for k, v in next, data do
                     --  local p = v[3] / descriptions[k].width -- or 3
                         local p = (v[1] / 1000) * factor * right
                         characters[k].right_protruding = p
                         if trace_protrusion then
-                            report_protrusions("rtbd -> %s -> U+%05X (%s) -> %0.03f (%s)",lookup,k,utfchar(k),p,concat(v," "))
+                            report_protrusions("rtbd -> %s -> %C -> %0.03f (% t)",lookup,k,p,v)
                         end
                     end
                 end
@@ -378,9 +387,15 @@ local function map_opbd_onto_protrusion(tfmdata,value,opbd)
             end
         end
     end
-    tfmdata.parameters.protrusion {
-        auto = true
-    }
+    local parameters = tfmdata.parameters
+    local protrusion = tfmdata.protrusion
+    if not protrusion then
+        parameters.protrusion = {
+            auto = true
+        }
+    else
+        protrusion.auto = true
+    end
 end
 
 -- The opbd test is just there because it was discussed on the
@@ -402,7 +417,7 @@ local function initializeprotrusion(tfmdata,value)
                     local left   = class.left   or 1
                     local right  = class.right  or 1
                     if trace_protrusion then
-                        report_protrusions("setting class %s, vector: %s, factor: %s, left: %s, right: %s",
+                        report_protrusions("setting class %a, vector %a, factor %a, left %a, right %a",
                             value,class.vector,factor,left,right)
                     end
                     local data = characters.data
@@ -443,10 +458,10 @@ local function initializeprotrusion(tfmdata,value)
                         end
                     end
                 elseif trace_protrusion then
-                    report_protrusions("unknown vector '%s' in class '%s",class.vector,value)
+                    report_protrusions("unknown vector %a in class %a",class.vector,value)
                 end
             elseif trace_protrusion then
-                report_protrusions("unknown class '%s'",value)
+                report_protrusions("unknown class %a",value)
             end
         end
     end
@@ -475,7 +490,7 @@ fonts.goodies.register("protrusions", function(...) return fonts.goodies.report(
 -- -- --
 
 local function initializenostackmath(tfmdata,value)
-    tfmdata.properties.no_stackmath = value and true
+    tfmdata.properties.nostackmath = value and true
 end
 
 registerotffeature {
@@ -487,20 +502,16 @@ registerotffeature {
     }
 }
 
-local function initializeitlc(tfmdata,value)
+local function initializeitlc(tfmdata,value) -- hm, always value
     if value then
-        -- the magic 40 and it formula come from Dohyun Kim
+        -- the magic 40 and it formula come from Dohyun Kim but we might need another guess
         local parameters = tfmdata.parameters
         local italicangle = parameters.italicangle
         if italicangle and italicangle ~= 0 then
-            local uwidth = (parameters.uwidth or 40)/2
-            for unicode, d in next, tfmdata.descriptions do
-                local it = d.boundingbox[3] - d.width + uwidth
-                if it ~= 0 then
-                    d.italic = it
-                end
-            end
-            tfmdata.properties.italic_correction = true
+            local properties = tfmdata.properties
+            local factor = tonumber(value) or 1
+            properties.hasitalics = true
+            properties.autoitalicamount = factor * (parameters.uwidth or 40)/2
         end
     end
 end
@@ -523,6 +534,30 @@ registerafmfeature {
     }
 }
 
+local function initializetextitalics(tfmdata,value) -- yes no delay
+    local delay = value == "delay"
+    tfmdata.properties.textitalics = delay and true or value
+    tfmdata.properties.delaytextitalics = delay
+end
+
+registerotffeature {
+    name        = "textitalics",
+    description = "use alternative text italic correction",
+    initializers = {
+        base = initializetextitalics,
+        node = initializetextitalics,
+    }
+}
+
+registerafmfeature {
+    name        = "textitalics",
+    description = "use alternative text italic correction",
+    initializers = {
+        base = initializetextitalics,
+        node = initializetextitalics,
+    }
+}
+
 -- slanting
 
 local function initializeslant(tfmdata,value)
@@ -534,7 +569,7 @@ local function initializeslant(tfmdata,value)
     elseif value < -1 then
         value = -1
     end
-    tfmdata.parameters.slant_factor = value
+    tfmdata.parameters.slantfactor = value
 end
 
 registerotffeature {
@@ -564,7 +599,7 @@ local function initializeextend(tfmdata,value)
     elseif value < -10 then
         value = -10
     end
-    tfmdata.parameters.extend_factor = value
+    tfmdata.parameters.extendfactor = value
 end
 
 registerotffeature {
@@ -585,6 +620,202 @@ registerafmfeature {
     }
 }
 
+-- For Wolfgang Schuster:
+--
+-- \definefontfeature[thisway][default][script=hang,language=zhs,dimensions={2,2,2}]
+-- \definedfont[file:kozminpr6nregular*thisway]
+--
+-- For the moment we don't mess with the descriptions.
+
+local function manipulatedimensions(tfmdata,key,value)
+    if type(value) == "string" and value ~= "" then
+        local characters = tfmdata.characters
+        local parameters = tfmdata.parameters
+        local emwidth = parameters.quad
+        local exheight = parameters.xheight
+        local spec = settings_to_array(value)
+        local width = (spec[1] or 0) * emwidth
+        local height = (spec[2] or 0) * exheight
+        local depth = (spec[3] or 0) * exheight
+        if width > 0 then
+            local resources = tfmdata.resources
+            local additions = { }
+            local private = resources.private
+            for unicode, old_c in next, characters do
+                local oldwidth = old_c.width
+                if oldwidth ~= width then
+                    -- Defining the tables in one step is more efficient
+                    -- than adding fields later.
+                    private = private + 1
+                    local new_c
+                    local commands = {
+                        { "right", (width - oldwidth) / 2 },
+                        { "slot", 1, private },
+                    }
+                    if height > 0 then
+                        if depth > 0 then
+                            new_c = {
+                                width    = width,
+                                height   = height,
+                                depth    = depth,
+                                commands = commands,
+                            }
+                        else
+                            new_c = {
+                                width    = width,
+                                height   = height,
+                                commands = commands,
+                            }
+                        end
+                    else
+                        if depth > 0 then
+                            new_c = {
+                                width    = width,
+                                depth    = depth,
+                                commands = commands,
+                            }
+                        else
+                            new_c = {
+                                width    = width,
+                                commands = commands,
+                            }
+                        end
+                    end
+                    setmetatableindex(new_c,old_c)
+                    characters[unicode] = new_c
+                    additions[private] = old_c
+                end
+            end
+            for k, v in next, additions do
+                characters[k] = v
+            end
+            resources.private = private
+        elseif height > 0 and depth > 0 then
+            for unicode, old_c in next, characters do
+                old_c.height = height
+                old_c.depth = depth
+            end
+        elseif height > 0 then
+            for unicode, old_c in next, characters do
+                old_c.height = height
+            end
+        elseif depth > 0 then
+            for unicode, old_c in next, characters do
+                old_c.depth = depth
+            end
+        end
+    end
+end
+
+registerotffeature {
+    name        = "dimensions",
+    description = "force dimensions",
+    manipulators = {
+        base = manipulatedimensions,
+        node = manipulatedimensions,
+    }
+}
+
+-- for zhichu chen (see mailing list archive): we might add a few more variants
+-- in due time
+--
+-- \definefontfeature[boxed][default][boundingbox=yes] % paleblue
+--
+-- maybe:
+--
+-- \definecolor[DummyColor][s=.75,t=.5,a=1] {\DummyColor test} \nopdfcompression
+--
+-- local gray  = { "special", "pdf: /Tr1 gs .75 g" }
+-- local black = { "special", "pdf: /Tr0 gs 0 g" }
+
+local push  = { "push" }
+local pop   = { "pop" }
+local gray  = { "special", "pdf: .75 g" }
+local black = { "special", "pdf: 0 g"   }
+
+local downcache = { } -- handy for huge cjk fonts
+local rulecache = { } -- handy for huge cjk fonts
+
+setmetatableindex(downcache,function(t,d)
+    local v = { "down", d }
+    t[d] = v
+    return v
+end)
+
+setmetatableindex(rulecache,function(t,h)
+    local v = { }
+    t[h] = v
+    setmetatableindex(v,function(t,w)
+        local v = { "rule", h, w }
+        t[w] = v
+        return v
+    end)
+    return v
+end)
+
+local function showboundingbox(tfmdata,key,value)
+    if value then
+        local vfspecials = backends.pdf.tables.vfspecials
+        local gray       = vfspecials and (vfspecials.rulecolors[value] or vfspecials.rulecolors.palegray) or gray
+        local characters = tfmdata.characters
+        local resources  = tfmdata.resources
+        local additions  = { }
+        local private = resources.private
+        for unicode, old_c in next, characters do
+            private = private + 1
+            local width  = old_c.width  or 0
+            local height = old_c.height or 0
+            local depth  = old_c.depth  or 0
+            local new_c
+            if depth == 0 then
+                new_c = {
+                    width    = width,
+                    height   = height,
+                    commands = {
+                        push,
+                        gray,
+                        rulecache[height][width],
+                        black,
+                        pop,
+                        { "slot", 1, private },
+                    }
+                }
+            else
+                new_c = {
+                    width    = width,
+                    height   = height,
+                    depth    = depth,
+                    commands = {
+                        push,
+                        downcache[depth],
+                        gray,
+                        rulecache[height+depth][width],
+                        black,
+                        pop,
+                        { "slot", 1, private },
+                    }
+                }
+            end
+            setmetatableindex(new_c,old_c)
+            characters[unicode] = new_c
+            additions[private] = old_c
+        end
+        for k, v in next, additions do
+            characters[k] = v
+        end
+        resources.private = private
+    end
+end
+
+registerotffeature {
+    name        = "boundingbox",
+    description = "show boundingbox",
+    manipulators = {
+        base = showboundingbox,
+        node = showboundingbox,
+    }
+}
+
 -- -- historic stuff, move from font-ota (handled differently, typo-rep)
 --
 -- local delete_node = nodes.delete
@@ -651,3 +882,65 @@ registerafmfeature {
 --         node = processformatters,
 --     }
 -- }
+
+-- a handy helper (might change or be moved to another namespace)
+
+local new_special = nodes.pool.special
+local new_glyph   = nodes.pool.glyph
+local hpack_node  = node.hpack
+
+function fonts.helpers.addprivate(tfmdata,name,characterdata)
+    local properties = tfmdata.properties
+    local privates = properties.privates
+    local lastprivate = properties.lastprivate
+    if lastprivate then
+        lastprivate = lastprivate + 1
+    else
+        lastprivate = 0xE000
+    end
+    if not privates then
+        privates = { }
+        properties.privates = privates
+    end
+    if name then
+        privates[name] = lastprivate
+    end
+    properties.lastprivate = lastprivate
+    tfmdata.characters[lastprivate] = characterdata
+    if properties.finalized then
+        properties.lateprivates = true
+    end
+    return lastprivate
+end
+
+function fonts.helpers.getprivatenode(tfmdata,name)
+    local properties = tfmdata.properties
+    local privates = properties and properties.privates
+    if privates then
+        local p = privates[name]
+        if p then
+            local char = tfmdata.characters[p]
+            local commands = char.commands
+            if commands then
+                local fake = hpack_node(new_special(commands[1][2]))
+                fake.width = char.width
+                fake.height = char.height
+                fake.depth = char.depth
+                return fake
+            else
+                -- todo: set current attribibutes
+                return new_glyph(properties.id,p)
+            end
+        end
+    end
+end
+
+function fonts.helpers.hasprivate(tfmdata,name)
+    local properties = tfmdata.properties
+    local privates = properties and properties.privates
+    return privates and privates[name] or false
+end
+
+function commands.getprivatechar(name)
+    context(fonts.helpers.getprivatenode(fontdata[font.current()],name))
+end
