@@ -22,11 +22,11 @@ local xmlfilter = xml.filter
 
 local type, setmetatable, getmetatable = type, setmetatable, getmetatable
 local insert, remove, fastcopy, concat = table.insert, table.remove, table.fastcopy, table.concat
-local gmatch, gsub, format = string.gmatch, string.gsub, string.format
+local gmatch, gsub, format, find, strip = string.gmatch, string.gsub, string.format, string.find, string.strip
 local utfbyte = utf.byte
 
 local function report(what,pattern,c,e)
-    report_xml("%s element '%s' (root: '%s', position: %s, index: %s, pattern: %s)",what,xmlname(e),xmlname(e.__p__),c,e.ni,pattern)
+    report_xml("%s element %a, root %a, position %a, index %a, pattern %a",what,xmlname(e),xmlname(e.__p__),c,e.ni,pattern)
 end
 
 local function withelements(e,handle,depth)
@@ -721,3 +721,91 @@ function xml.setcdata(e,str) -- also setcomment
         dt      = { str },
     } }
 end
+
+-- maybe helpers like this will move to an autoloader
+
+function xml.separate(x,pattern)
+    local collected = xmlapplylpath(x,pattern)
+    if collected then
+        for c=1,#collected do
+            local e = collected[c]
+            local d = e.dt
+            if d == x then
+                report_xml("warning: xml.separate changes root")
+                x = d
+            end
+            local t, n = { "\n" }, 1
+            local i, nd = 1, #d
+            while i <= nd do
+                while i <= nd do
+                    local di = d[i]
+                    if type(di) == "string" then
+                        if di == "\n" or find(di,"^%s+$") then -- first test is speedup
+                            i = i + 1
+                        else
+                            d[i] = strip(di)
+                            break
+                        end
+                    else
+                        break
+                    end
+                end
+                if i > nd then
+                    break
+                end
+                t[n+1] = "\n"
+                t[n+2] = d[i]
+                t[n+3] = "\n"
+                n = n + 3
+                i = i + 1
+            end
+            t[n+1] = "\n"
+            setmetatable(t,getmetatable(d))
+            e.dt = t
+        end
+    end
+    return x
+end
+
+--
+
+local helpers = xml.helpers or { }
+xml.helpers   = helpers
+
+local function normal(e,action)
+    local edt = e.dt
+    if edt then
+        for i=1,#edt do
+            local str = edt[i]
+            if type(str) == "string" and str ~= "" then
+                edt[i] = action(str)
+            end
+        end
+    end
+end
+
+local function recurse(e,action)
+    local edt = e.dt
+    if edt then
+        for i=1,#edt do
+            local str = edt[i]
+            if type(str) ~= "string" then
+                recurse(str,action,recursive)
+            elseif str ~= "" then
+                edt[i] = action(str)
+            end
+        end
+    end
+end
+
+function helpers.recursetext(collected,action,recursive)
+    if recursive then
+        for i=1,#collected do
+            recurse(collected[i],action)
+        end
+    else
+        for i=1,#collected do
+           normal(collected[i],action)
+        end
+    end
+end
