@@ -17,6 +17,7 @@ return {
         "zeropoint", "onepoint", "halfapoint", "onebasepoint", "maxdimen", "scaledpoint", "thousandpoint",
         "points", "halfpoint",
         "zeroskip",
+        "zeromuskip", "onemuskip",
         "pluscxxvii", "pluscxxviii", "pluscclv", "pluscclvi",
         "normalpagebox",
         --        --
@@ -94,13 +95,15 @@ return {
         "startcomponent", "stopcomponent", "component",
         "startproduct", "stopproduct", "product",
         "startproject", "stopproject", "project",
-        "starttext", "stoptext", "startnotext", "stopnotext","startdocument", "stopdocument", "documentvariable",
-        "startmodule", "stopmodule", "usemodule",
+        "starttext", "stoptext", "startnotext", "stopnotext","startdocument", "stopdocument", "documentvariable", "setupdocument",
+        "startmodule", "stopmodule", "usemodule", "usetexmodule", "useluamodule",
         --
         "startTEXpage", "stopTEXpage",
     --  "startMPpage", "stopMPpage", -- already catched by nested lexer
         --
-        "enablemode", "disablemode", "preventmode", "pushmode", "popmode",
+        "enablemode", "disablemode", "preventmode",
+        "globalenablemode", "globaldisablemode", "globalpreventmode",
+        "pushmode", "popmode",
         --
         "typescriptone", "typescripttwo", "typescriptthree", "mathsizesuffix",
         --
@@ -111,6 +114,8 @@ return {
         "constantnumber", "constantnumberargument", "constantdimen", "constantdimenargument", "constantemptyargument",
         --
         "continueifinputfile",
+        --
+        "luastringsep", "!!bs", "!!es",
     },
     ["helpers"] = {
         --
@@ -122,8 +127,10 @@ return {
         "startlocalsetups", "stoplocalsetups",
         "starttexdefinition", "stoptexdefinition",
         "starttexcode", "stoptexcode",
+        "startcontextcode", "stopcontextcode",
         --
         "doifsetupselse", "doifsetups", "doifnotsetups", "setup", "setups", "texsetup", "xmlsetup", "luasetup", "directsetup",
+        "doifelsecommandhandler","doifnotcommandhandler","doifcommandhandler",
         --
         "newmode", "setmode", "resetmode",
         "newsystemmode", "setsystemmode", "resetsystemmode", "pushsystemmode", "popsystemmode",
@@ -133,14 +140,26 @@ return {
         "newlanguage", "newfamily", "newfam", "newhelp", -- not used
         --
         "then",
+        "begcsname",
         --
-        "donothing", "dontcomplain",
+        "strippedcsname",
+        --
+        "firstargumentfalse", "firstargumenttrue",
+        "secondargumentfalse", "secondargumenttrue",
+        "thirdargumentfalse", "thirdargumenttrue",
+        "fourthargumentfalse", "fourthargumenttrue",
+        "fifthargumentfalse", "fifthsargumenttrue",
+        "sixthargumentfalse", "sixtsargumenttrue",
+        --
+        "doglobal", "dodoglobal", "redoglobal", "resetglobal",
+        --
+        "donothing", "dontcomplain", "forgetall",
         --
         "donetrue", "donefalse",
         --
         "htdp",
         "unvoidbox",
-        "vfilll",
+        "hfilll", "vfilll",
         --
         "mathbox", "mathlimop", "mathnolop", "mathnothing", "mathalpha",
         --
@@ -158,6 +177,8 @@ return {
         "ruledhglue", "ruledvglue", "normalhglue", "normalvglue",
         "ruledpenalty",
         --
+        "filledhboxb", "filledhboxr", "filledhboxg", "filledhboxc", "filledhboxm", "filledhboxy", "filledhboxk",
+        --
         "scratchcounter", "globalscratchcounter",
         "scratchdimen", "globalscratchdimen",
         "scratchskip", "globalscratchskip",
@@ -165,10 +186,15 @@ return {
         "scratchtoks", "globalscratchtoks",
         "scratchbox", "globalscratchbox",
         --
+        "availablehsize", "localhsize", "setlocalhsize",
+        --
         "nextbox", "dowithnextbox", "dowithnextboxcs", "dowithnextboxcontent", "dowithnextboxcontentcs",
         --
         "scratchwidth", "scratchheight", "scratchdepth", "scratchoffset", "scratchdistance",
         "scratchhsize", "scratchvsize",
+        "scratchxoffset", "scratchyoffset", "scratchhoffset", "scratchvoffset",
+        "scratchxposition", "scratchyposition",
+        "scratchtopoffset", "scratchbottomoffset", "scratchleftoffset", "scratchrightoffset",
         --
         "scratchcounterone", "scratchcountertwo", "scratchcounterthree",
         "scratchdimenone", "scratchdimentwo", "scratchdimenthree",
@@ -176,6 +202,10 @@ return {
         "scratchmuskipone", "scratchmuskiptwo", "scratchmuskipthree",
         "scratchtoksone", "scratchtokstwo", "scratchtoksthree",
         "scratchboxone", "scratchboxtwo", "scratchboxthree",
+        "scratchnx", "scratchny", "scratchmx", "scratchmy",
+        "scratchunicode",
+        --
+        "scratchleftskip", "scratchrightskip", "scratchtopskip", "scratchbottomskip",
         --
         "doif", "doifnot", "doifelse",
         "doifinset", "doifnotinset", "doifinsetelse",
@@ -184,33 +214,42 @@ return {
         "doifelsevalue", "doifvalue", "doifnotvalue",
         "doifnothing", "doifsomething", "doifelsenothing", "doifsomethingelse",
         "doifvaluenothing", "doifvaluesomething", "doifelsevaluenothing",
-        "doifdimensionelse", "doifnumberelse",
+        "doifdimensionelse", "doifnumberelse", "doifnumber", "doifnotnumber",
         "doifcommonelse", "doifcommon", "doifnotcommon",
         "doifinstring", "doifnotinstring", "doifinstringelse",
-        "doifassignmentelse",
+        "doifassignmentelse", "docheckassignment",
         --
         "tracingall", "tracingnone", "loggingall",
         --
-        "appendtoks", "prependtoks", "appendtotoks", "prependtotoks", "to",
+        "removetoks", "appendtoks", "prependtoks", "appendtotoks", "prependtotoks", "to",
         --
-        "endgraf", "empty", "null", "space", "quad", "enspace", "obeyspaces", "obeylines", "normalspace",
+        "endgraf", "endpar", "everyendpar", "reseteverypar", "finishpar", "empty", "null", "space", "quad", "enspace",
+        "obeyspaces", "obeylines", "obeyedspace", "obeyedline",
+        "normalspace",
         --
         "executeifdefined",
         --
         "singleexpandafter", "doubleexpandafter", "tripleexpandafter",
         --
-        "dontleavehmode", "removelastspace", "removeunwantedspaces",
+        "dontleavehmode", "removelastspace", "removeunwantedspaces", "keepunwantedspaces",
         --
-        "wait", "writestatus", "define", "redefine",
+        "wait", "writestatus", "define", "defineexpandable", "redefine",
         --
-        "setmeasure", "setemeasure", "setgmeasure", "setxmeasure", "definemeasure", "measure",
+        "setmeasure", "setemeasure", "setgmeasure", "setxmeasure", "definemeasure", "freezemeasure", "measure", "measured",
         --
-        "getvalue", "setvalue", "setevalue", "setgvalue", "setxvalue", "letvalue", "letgvalue",
+        "installcorenamespace",
+        --
+        "getvalue", "getuvalue", "setvalue", "setevalue", "setgvalue", "setxvalue", "letvalue", "letgvalue",
         "resetvalue", "undefinevalue", "ignorevalue",
         "setuvalue", "setuevalue", "setugvalue", "setuxvalue",
-        "globallet", "glet",
+        --
+        "globallet", "glet", "udef", "ugdef", "uedef", "uxdef", "checked", "unique",
+        --
         "getparameters", "geteparameters", "getgparameters", "getxparameters", "forgetparameters", "copyparameters",
         --
+        "getdummyparameters", "dummyparameter", "directdummyparameter", "setdummyparameter", "letdummyparameter",
+        "usedummystyleandcolor", "usedummystyleparameter", "usedummycolorparameter",
+        --
         "processcommalist", "processcommacommand", "quitcommalist", "quitprevcommalist",
         "processaction", "processallactions", "processfirstactioninset", "processallactionsinset",
         --
@@ -228,14 +267,16 @@ return {
         "gobbleoneargument", "gobbletwoarguments", "gobblethreearguments", "gobblefourarguments", "gobblefivearguments", "gobblesixarguments", "gobblesevenarguments", "gobbleeightarguments", "gobbleninearguments", "gobbletenarguments",
         "gobbleoneoptional", "gobbletwooptionals", "gobblethreeoptionals", "gobblefouroptionals", "gobblefiveoptionals",
         --
-        "dorecurse", "doloop", "exitloop", "dostepwiserecurse", "recurselevel", "recursedepth", "dofastloopcs",
+        "dorecurse", "doloop", "exitloop", "dostepwiserecurse", "recurselevel", "recursedepth", "dofastloopcs", "dowith",
         --
         "newconstant", "setnewconstant", "newconditional", "settrue", "setfalse", "setconstant",
         "newmacro", "setnewmacro", "newfraction",
+        "newsignal",
         --
         "dosingleempty", "dodoubleempty", "dotripleempty", "doquadrupleempty", "doquintupleempty", "dosixtupleempty", "doseventupleempty",
-        "dosingleargument", "dodoubleargument", "dotripleargument", "doquadrupleargument",
+        "dosingleargument", "dodoubleargument", "dotripleargument", "doquadrupleargument", "doquintupleargument", "dosixtupleargument", "doseventupleargument",
         "dosinglegroupempty", "dodoublegroupempty", "dotriplegroupempty", "doquadruplegroupempty", "doquintuplegroupempty",
+        "permitspacesbetweengroups", "dontpermitspacesbetweengroups",
         --
         "nopdfcompression", "maximumpdfcompression", "normalpdfcompression",
         --
@@ -245,6 +286,62 @@ return {
         --
         "startnointerference", "stopnointerference",
         --
-        "strut", "setstrut", "strutbox", "strutht", "strutdp", "strutwd", "begstrut", "endstrut",
+        "twodigits","threedigits",
+        --
+        "strut", "setstrut", "strutbox", "strutht", "strutdp", "strutwd", "struthtdp", "begstrut", "endstrut", "lineheight",
+        --
+        "ordordspacing", "ordopspacing", "ordbinspacing", "ordrelspacing",
+        "ordopenspacing", "ordclosespacing", "ordpunctspacing", "ordinnerspacing",
+        --
+        "opordspacing", "opopspacing", "opbinspacing", "oprelspacing",
+        "opopenspacing", "opclosespacing", "oppunctspacing", "opinnerspacing",
+        --
+        "binordspacing", "binopspacing", "binbinspacing", "binrelspacing",
+        "binopenspacing", "binclosespacing", "binpunctspacing", "bininnerspacing",
+        --
+        "relordspacing", "relopspacing", "relbinspacing", "relrelspacing",
+        "relopenspacing", "relclosespacing", "relpunctspacing", "relinnerspacing",
+        --
+        "openordspacing", "openopspacing", "openbinspacing", "openrelspacing",
+        "openopenspacing", "openclosespacing", "openpunctspacing", "openinnerspacing",
+        --
+        "closeordspacing", "closeopspacing", "closebinspacing", "closerelspacing",
+        "closeopenspacing", "closeclosespacing", "closepunctspacing", "closeinnerspacing",
+        --
+        "punctordspacing", "punctopspacing", "punctbinspacing", "punctrelspacing",
+        "punctopenspacing", "punctclosespacing", "punctpunctspacing", "punctinnerspacing",
+        --
+        "innerordspacing", "inneropspacing", "innerbinspacing", "innerrelspacing",
+        "inneropenspacing", "innerclosespacing", "innerpunctspacing", "innerinnerspacing",
+        --
+        "normalreqno",
+        --
+        "startimath", "stopimath", "normalstartimath", "normalstopimath",
+        "startdmath", "stopdmath", "normalstartdmath", "normalstopdmath",
+        --
+        "uncramped", "cramped", "triggermathstyle", "mathstylefont", "mathsmallstylefont", "mathstyleface", "mathsmallstyleface", "mathstylecommand", "mathpalette",
+        "mathstylehbox", "mathstylevbox", "mathstylevcenter", "mathstylevcenteredhbox", "mathstylevcenteredvbox",
+        "mathtext", "setmathsmalltextbox", "setmathtextbox",
+        --
+        "triggerdisplaystyle", "triggertextstyle", "triggerscriptstyle", "triggerscriptscriptstyle",
+        "triggeruncrampedstyle", "triggercrampedstyle",
+        "triggersmallstyle", "triggeruncrampedsmallstyle", "triggercrampedsmallstyle",
+        "triggerbigstyle", "triggeruncrampedbigstyle", "triggercrampedbigstyle",
+        --
+        "luaexpr", "expdoifelse", "expdoif", "expdoifnot", "expdoifcommonelse", "expdoifinsetelse",
+        --
+        "ctxdirectlua", "ctxlatelua", "ctxsprint", "ctxwrite", "ctxcommand", "ctxdirectcommand", "ctxlatecommand", "ctxreport",
+        "ctxlua", "luacode", "lateluacode", "directluacode",
+        "registerctxluafile", "ctxloadluafile",
+        "luaversion", "luamajorversion", "luaminorversion",
+        "ctxluacode", "luaconditional", "luaexpanded",
+        "startluaparameterset", "stopluaparameterset", "luaparameterset",
+        "definenamedlua",
+        "obeylualines", "obeyluatokens",
+        "startluacode", "stopluacode", "startlua", "stoplua",
+        --
+        "carryoverpar",
+        --
+        "Umathbotaccent",
     }
 }
