@@ -3,15 +3,16 @@ if not modules then modules = { } end modules ['node-inj'] = {
     comment   = "companion to node-ini.mkiv",
     author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
     copyright = "PRAGMA ADE / ConTeXt Development Team",
-    license   = "see context related readme files"
+    license   = "see context related readme files",
 }
 
 -- This is very experimental (this will change when we have luatex > .50 and
 -- a few pending thingies are available. Also, Idris needs to make a few more
 -- test fonts. Btw, future versions of luatex will have extended glyph properties
--- that can be of help.
+-- that can be of help. Some optimizations can go away when we have faster machines.
 
 local next = next
+local utfchar = utf.char
 
 local trace_injections = false  trackers.register("nodes.injections", function(v) trace_injections = v end)
 
@@ -27,46 +28,32 @@ local injections         = nodes.injections
 
 local nodecodes          = nodes.nodecodes
 local glyph_code         = nodecodes.glyph
+local kern_code          = nodecodes.kern
 local nodepool           = nodes.pool
 local newkern            = nodepool.kern
 
 local traverse_id        = node.traverse_id
-local unset_attribute    = node.unset_attribute
-local has_attribute      = node.has_attribute
-local set_attribute      = node.set_attribute
-local copy_node          = node.copy
 local insert_node_before = node.insert_before
 local insert_node_after  = node.insert_after
 
-local markbase = attributes.private('markbase')
-local markmark = attributes.private('markmark')
-local markdone = attributes.private('markdone')
-local cursbase = attributes.private('cursbase')
-local curscurs = attributes.private('curscurs')
-local cursdone = attributes.private('cursdone')
-local kernpair = attributes.private('kernpair')
-local ligacomp = attributes.private('ligacomp')
-local fontkern = attributes.private('fontkern')
-
-if context then
-
-    local kern = nodes.pool.register(newkern())
-
-    set_attribute(kern,fontkern,1) -- we can have several, attributes are shared
-
-    newkern = function(k)
-        local c = copy_node(kern)
-        c.kern = k
-        return c
-    end
-
-end
+local a_kernpair = attributes.private('kernpair')
+local a_ligacomp = attributes.private('ligacomp')
+local a_markbase = attributes.private('markbase')
+local a_markmark = attributes.private('markmark')
+local a_markdone = attributes.private('markdone')
+local a_cursbase = attributes.private('cursbase')
+local a_curscurs = attributes.private('curscurs')
+local a_cursdone = attributes.private('cursdone')
 
 -- This injector has been tested by Idris Samawi Hamid (several arabic fonts as well as
 -- the rather demanding Husayni font), Khaled Hosny (latin and arabic) and Kaj Eigner
 -- (arabic, hebrew and thai) and myself (whatever font I come across). I'm pretty sure
 -- that this code is not 100% okay but examples are needed to figure things out.
 
+function injections.installnewkern(nk)
+    newkern = nk or newkern
+end
+
 local cursives = { }
 local marks    = { }
 local kerns    = { }
@@ -82,8 +69,8 @@ function injections.setcursive(start,nxt,factor,rlmode,exit,entry,tfmstart,tfmne
     local dx, dy = factor*(exit[1]-entry[1]), factor*(exit[2]-entry[2])
     local ws, wn = tfmstart.width, tfmnext.width
     local bound = #cursives + 1
-    set_attribute(start,cursbase,bound)
-    set_attribute(nxt,curscurs,bound)
+    start[a_cursbase] = bound
+    nxt[a_curscurs] = bound
     cursives[bound] = { rlmode, dx, dy, ws, wn }
     return dx, dy, bound
 end
@@ -92,14 +79,14 @@ function injections.setpair(current,factor,rlmode,r2lflag,spec,tfmchr)
     local x, y, w, h = factor*spec[1], factor*spec[2], factor*spec[3], factor*spec[4]
     -- dy = y - h
     if x ~= 0 or w ~= 0 or y ~= 0 or h ~= 0 then
-        local bound = has_attribute(current,kernpair)
+        local bound = current[a_kernpair]
         if bound then
             local kb = kerns[bound]
             -- inefficient but singles have less, but weird anyway, needs checking
             kb[2], kb[3], kb[4], kb[5] = (kb[2] or 0) + x, (kb[3] or 0) + y, (kb[4] or 0)+ w, (kb[5] or 0) + h
         else
             bound = #kerns + 1
-            set_attribute(current,kernpair,bound)
+            current[a_kernpair] = bound
             kerns[bound] = { rlmode, x, y, w, h, r2lflag, tfmchr.width }
         end
         return x, y, w, h, bound
@@ -111,7 +98,7 @@ function injections.setkern(current,factor,rlmode,x,tfmchr)
     local dx = factor*x
     if dx ~= 0 then
         local bound = #kerns + 1
-        set_attribute(current,kernpair,bound)
+        current[a_kernpair] = bound
         kerns[bound] = { rlmode, dx }
         return dx, bound
     else
@@ -121,27 +108,27 @@ end
 
 function injections.setmark(start,base,factor,rlmode,ba,ma,index) -- ba=baseanchor, ma=markanchor
     local dx, dy = factor*(ba[1]-ma[1]), factor*(ba[2]-ma[2])     -- the index argument is no longer used but when this
-    local bound = has_attribute(base,markbase)                    -- fails again we should pass it
-local index = 1
+    local bound = base[a_markbase]                    -- fails again we should pass it
+    local index = 1
     if bound then
         local mb = marks[bound]
         if mb then
          -- if not index then index = #mb + 1 end
-index = #mb + 1
+            index = #mb + 1
             mb[index] = { dx, dy, rlmode }
-            set_attribute(start,markmark,bound)
-            set_attribute(start,markdone,index)
+            start[a_markmark] = bound
+            start[a_markdone] = index
             return dx, dy, bound
         else
-            report_injections("possible problem, U+%05X is base mark without data (id: %s)",base.char,bound)
+            report_injections("possible problem, %U is base mark without data (id %a)",base.char,bound)
         end
     end
 --     index = index or 1
     index = index or 1
     bound = #marks + 1
-    set_attribute(base,markbase,bound)
-    set_attribute(start,markmark,bound)
-    set_attribute(start,markdone,index)
+    base[a_markbase] = bound
+    start[a_markmark] = bound
+    start[a_markdone] = index
     marks[bound] = { [index] = { dx, dy, rlmode } }
     return dx, dy, bound
 end
@@ -154,44 +141,45 @@ local function trace(head)
     report_injections("begin run")
     for n in traverse_id(glyph_code,head) do
         if n.subtype < 256 then
-            local kp = has_attribute(n,kernpair)
-            local mb = has_attribute(n,markbase)
-            local mm = has_attribute(n,markmark)
-            local md = has_attribute(n,markdone)
-            local cb = has_attribute(n,cursbase)
-            local cc = has_attribute(n,curscurs)
-            report_injections("char U+%05X, font=%s",n.char,n.font)
+            local kp = n[a_kernpair]
+            local mb = n[a_markbase]
+            local mm = n[a_markmark]
+            local md = n[a_markdone]
+            local cb = n[a_cursbase]
+            local cc = n[a_curscurs]
+            local char = n.char
+            report_injections("font %s, char %U, glyph %c",n.font,char,char)
             if kp then
                 local k = kerns[kp]
                 if k[3] then
-                    report_injections("  pairkern: dir=%s, x=%s, y=%s, w=%s, h=%s",dir(k[1]),k[2] or "?",k[3] or "?",k[4] or "?",k[5] or "?")
+                    report_injections("  pairkern: dir %a, x %p, y %p, w %p, h %p",dir(k[1]),k[2],k[3],k[4],k[5])
                 else
-                    report_injections("  kern: dir=%s, dx=%s",dir(k[1]),k[2] or "?")
+                    report_injections("  kern: dir %a, dx %p",dir(k[1]),k[2])
                 end
             end
             if mb then
-                report_injections("  markbase: bound=%s",mb)
+                report_injections("  markbase: bound %a",mb)
             end
             if mm then
                 local m = marks[mm]
                 if mb then
                     local m = m[mb]
                     if m then
-                        report_injections("  markmark: bound=%s, index=%s, dx=%s, dy=%s",mm,md or "?",m[1] or "?",m[2] or "?")
+                        report_injections("  markmark: bound %a, index %a, dx %p, dy %p",mm,md,m[1],m[2])
                     else
-                        report_injections("  markmark: bound=%s, missing index",mm)
+                        report_injections("  markmark: bound %a, missing index",mm)
                     end
                 else
                     m = m[1]
-                    report_injections("  markmark: bound=%s, dx=%s, dy=%s",mm,m and m[1] or "?",m and m[2] or "?")
+                    report_injections("  markmark: bound %a, dx %p, dy %p",mm,m and m[1],m and m[2])
                 end
             end
             if cb then
-                report_injections("  cursbase: bound=%s",cb)
+                report_injections("  cursbase: bound %a",cb)
             end
             if cc then
                 local c = cursives[cc]
-                report_injections("  curscurs: bound=%s, dir=%s, dx=%s, dy=%s",cc,dir(c[1]),c[2] or "?",c[3] or "?")
+                report_injections("  curscurs: bound %a, dir %a, dx %p, dy %p",cc,dir(c[1]),c[2],c[3])
             end
         end
     end
@@ -204,6 +192,25 @@ end
 -- We can have a fast test on a font being processed, so we can check faster for marks etc
 -- but I'll make a context variant anyway.
 
+local function show_result(head)
+    local current = head
+    local skipping = false
+    while current do
+        local id = current.id
+        if id == glyph_code then
+            report_injections("char: %C, width %p, xoffset %p, yoffset %p",current.char,current.width,current.xoffset,current.yoffset)
+            skipping = false
+        elseif id == kern_code then
+            report_injections("kern: %p",current.kern)
+            skipping = false
+        elseif not skipping then
+            report_injections()
+            skipping = true
+        end
+        current = current.next
+    end
+end
+
 function injections.handler(head,where,keep)
     local has_marks, has_cursives, has_kerns = next(marks), next(cursives), next(kerns)
     if has_marks or has_cursives then
@@ -225,7 +232,7 @@ function injections.handler(head,where,keep)
                     if tm then
                         mk[n] = tm[n.char]
                     end
-                    local k = has_attribute(n,kernpair)
+                    local k = n[a_kernpair]
                     if k then
                         local kk = kerns[k]
                         if kk then
@@ -274,9 +281,9 @@ function injections.handler(head,where,keep)
                 for i=1,nofvalid do -- valid == glyphs
                     local n = valid[i]
                     if not mk[n] then
-                        local n_cursbase = has_attribute(n,cursbase)
+                        local n_cursbase = n[a_cursbase]
                         if p_cursbase then
-                            local n_curscurs = has_attribute(n,curscurs)
+                            local n_curscurs = n[a_curscurs]
                             if p_cursbase == n_curscurs then
                                 local c = cursives[n_curscurs]
                                 if c then
@@ -337,46 +344,53 @@ function injections.handler(head,where,keep)
             if has_marks then
                 for i=1,nofvalid do
                     local p = valid[i]
-                    local p_markbase = has_attribute(p,markbase)
+                    local p_markbase = p[a_markbase]
                     if p_markbase then
                         local mrks = marks[p_markbase]
                         local nofmarks = #mrks
                         for n in traverse_id(glyph_code,p.next) do
-                            local n_markmark = has_attribute(n,markmark)
+                            local n_markmark = n[a_markmark]
                             if p_markbase == n_markmark then
-                                local index = has_attribute(n,markdone) or 1
+                                local index = n[a_markdone] or 1
                                 local d = mrks[index]
                                 if d then
                                     local rlmode = d[3]
-                                    if rlmode and rlmode >= 0 then
-                                        -- new per 2010-10-06, width adapted per 2010-02-03
-                                        -- we used to negate the width of marks because in tfm
-                                        -- that makes sense but we no longer do that so as a
-                                        -- consequence the sign of p.width was changed (we need
-                                        -- to keep an eye on it as we don't have that many fonts
-                                        -- that enter this branch .. I'm still not sure if this
-                                        -- one is right
-                                        local k = wx[p]
-                                        if k then
-                                            n.xoffset = p.xoffset + p.width + d[1] - k[2]
+                                    --
+                                    local k = wx[p]
+                                    if k then
+                                        local x = k[2]
+                                        local w = k[4]
+                                        if w then
+                                            if rlmode and rlmode >= 0 then
+                                                -- kern(x) glyph(p) kern(w-x) mark(n)
+                                                n.xoffset = p.xoffset - p.width + d[1] - (w-x)
+                                            else
+                                                -- kern(w-x) glyph(p) kern(x) mark(n)
+                                                n.xoffset = p.xoffset - d[1] - x
+                                            end
                                         else
-                                         -- n.xoffset = p.xoffset + p.width + d[1]
-                                            -- lucida U\char"032F (default+mark)
-                                            n.xoffset = p.xoffset - p.width + d[1] -- 01-05-2011
+                                            if rlmode and rlmode >= 0 then
+                                                -- okay for husayni
+                                                n.xoffset = p.xoffset - p.width + d[1]
+                                            else
+                                                -- needs checking: is x ok here?
+                                                n.xoffset = p.xoffset - d[1] - x
+                                            end
                                         end
                                     else
-                                        local k = wx[p]
-                                        if k then
-                                            n.xoffset = p.xoffset - d[1] - k[2]
+                                        if rlmode and rlmode >= 0 then
+                                            n.xoffset = p.xoffset - p.width + d[1]
                                         else
                                             n.xoffset = p.xoffset - d[1]
                                         end
                                     end
+                                    --                                    --
                                     if mk[p] then
                                         n.yoffset = p.yoffset + d[2]
                                     else
                                         n.yoffset = n.yoffset + p.yoffset + d[2]
                                     end
+                                    --
                                     if nofmarks == 1 then
                                         break
                                     else
@@ -397,23 +411,24 @@ function injections.handler(head,where,keep)
             if next(wx) then
                 for n, k in next, wx do
                  -- only w can be nil (kernclasses), can be sped up when w == nil
-                    local x, w = k[2] or 0, k[4]
+                    local x = k[2]
+                    local w = k[4]
                     if w then
                         local rl = k[1] -- r2l = k[6]
                         local wx = w - x
                         if rl < 0 then	-- KE: don't use r2l here
                             if wx ~= 0 then
-                                insert_node_before(head,n,newkern(wx))
+                                insert_node_before(head,n,newkern(wx)) -- type 0/2
                             end
                             if x ~= 0 then
-                                insert_node_after (head,n,newkern(x))
+                                insert_node_after (head,n,newkern(x))  -- type 0/2
                             end
                         else
                             if x ~= 0 then
-                                insert_node_before(head,n,newkern(x))
+                                insert_node_before(head,n,newkern(x))  -- type 0/2
                             end
                             if wx ~= 0 then
-                                insert_node_after(head,n,newkern(wx))
+                                insert_node_after (head,n,newkern(wx)) -- type 0/2
                             end
                         end
                     elseif x ~= 0 then
@@ -421,7 +436,7 @@ function injections.handler(head,where,keep)
                         -- uses kernclasses between glyphs so we're probably safe (KE has a
                         -- problematic font where marks interfere with rl < 0 in the previous
                         -- case)
-                        insert_node_before(head,n,newkern(x))
+                        insert_node_before(head,n,newkern(x)) -- a real font kern, type 0
                     end
                 end
             end
@@ -430,9 +445,9 @@ function injections.handler(head,where,keep)
                     if k ~= 0 then
                         local rln = rl[n]
                         if rln and rln < 0 then
-                            insert_node_before(head,n,newkern(-k))
+                            insert_node_before(head,n,newkern(-k)) -- type 0/2
                         else
-                            insert_node_before(head,n,newkern(k))
+                            insert_node_before(head,n,newkern(k))  -- type 0/2
                         end
                     end
                 end
@@ -440,6 +455,9 @@ function injections.handler(head,where,keep)
             if not keep then
                 kerns = { }
             end
+         -- if trace_injections then
+         --     show_result(head)
+         -- end
             return head, true
         elseif not keep then
             kerns, cursives, marks = { }, { }, { }
@@ -450,7 +468,7 @@ function injections.handler(head,where,keep)
         end
         for n in traverse_id(glyph_code,head) do
             if n.subtype < 256 then
-                local k = has_attribute(n,kernpair)
+                local k = n[a_kernpair]
                 if k then
                     local kk = kerns[k]
                     if kk then
@@ -490,6 +508,9 @@ function injections.handler(head,where,keep)
         if not keep then
             kerns = { }
         end
+     -- if trace_injections then
+     --     show_result(head)
+     -- end
         return head, true
     else
         -- no tracing needed
