@@ -1,6 +1,6 @@
-if not modules then modules = { } end modules ['node-bck'] = {
+if not modules then modules = { } end modules ['node-ref'] = {
     version   = 1.001,
-    comment   = "companion to node-bck.mkiv",
+    comment   = "companion to node-ref.mkiv",
     author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
     copyright = "PRAGMA ADE / ConTeXt Development Team",
     license   = "see context related readme files"
@@ -16,6 +16,8 @@ if not modules then modules = { } end modules ['node-bck'] = {
 
 -- is grouplevel still used?
 
+local format = string.format
+
 local allocate, mark = utilities.storage.allocate, utilities.storage.mark
 
 local cleanupreferences, cleanupdestinations = false, true
@@ -66,29 +68,28 @@ local nodepool         = nodes.pool
 
 local new_kern         = nodepool.kern
 
-local has_attribute    = node.has_attribute
 local traverse         = node.traverse
 local find_node_tail   = node.tail or node.slide
 local tosequence       = nodes.tosequence
 
-local function dimensions(parent,start,stop)
-    stop = stop and stop.next
-    if parent then
-        if stop then
-            return list_dimensions(parent.glue_set,parent.glue_sign,parent.glue_order,start,stop)
-        else
-            return list_dimensions(parent.glue_set,parent.glue_sign,parent.glue_order,start)
-        end
-    else
-        if stop then
-            return list_dimensions(start,stop)
-        else
-            return list_dimensions(start)
-        end
-    end
-end
-
---~ more compact
+-- local function dimensions(parent,start,stop)
+--     stop = stop and stop.next
+--     if parent then
+--         if stop then
+--             return list_dimensions(parent.glue_set,parent.glue_sign,parent.glue_order,start,stop)
+--         else
+--             return list_dimensions(parent.glue_set,parent.glue_sign,parent.glue_order,start)
+--         end
+--     else
+--         if stop then
+--             return list_dimensions(start,stop)
+--         else
+--             return list_dimensions(start)
+--         end
+--     end
+-- end
+--
+-- -- more compact
 
 local function dimensions(parent,start,stop)
     if parent then
@@ -102,21 +103,21 @@ end
 
 local function inject_range(head,first,last,reference,make,stack,parent,pardir,txtdir)
     local width, height, depth = dimensions(parent,first,last)
-    if pardir == "TRT" or txtdir == "+TRT" then
+    if txtdir == "+TRT" or (txtdir == "===" and pardir == "TRT") then -- KH: textdir == "===" test added
         width = - width
     end
     local result, resolved = make(width,height,depth,reference)
     if result and resolved then
         if head == first then
             if trace_backend then
-                report_area("head: %04i %s %s %s => w=%s, h=%s, d=%s, c=%s",reference,pardir or "---",txtdir or "----",tosequence(first,last,true),width,height,depth,resolved)
+                report_area("head: %04i %s %s %s => w=%p, h=%p, d=%p, c=%s",reference,pardir or "---",txtdir or "----",tosequence(first,last,true),width,height,depth,resolved)
             end
             result.next = first
             first.prev = result
             return result, last
         else
             if trace_backend then
-                report_area("middle: %04i %s %s => w=%s, h=%s, d=%s, c=%s",reference,pardir or "---",txtdir or "----",tosequence(first,last,true),width,height,depth,resolved)
+                report_area("middle: %04i %s %s => w=%p, h=%p, d=%p, c=%s",reference,pardir or "---",txtdir or "----",tosequence(first,last,true),width,height,depth,resolved)
             end
             local prev = first.prev
             if prev then
@@ -178,9 +179,10 @@ local function inject_list(id,current,reference,make,stack,pardir,txtdir)
         width = - width
     end
     local result, resolved = make(width,height,depth,reference)
+    -- todo: only when width is ok
     if result and resolved then
         if trace_backend then
-            report_area("box: %04i %s %s: w=%s, h=%s, d=%s, c=%s",reference,pardir or "---",txtdir or "----",width,height,depth,resolved)
+            report_area("box: %04i %s %s: w=%p, h=%p, d=%p, c=%s",reference,pardir or "---",txtdir or "----",width,height,depth,resolved)
         end
         if not first then
             current.list = result
@@ -209,21 +211,12 @@ local function inject_areas(head,attribute,make,stack,done,skip,parent,pardir,tx
         txtdir = txtdir or "==="
         while current do
             local id = current.id
-            local r = has_attribute(current,attribute)
-            if id == whatsit_code then
-                local subtype = current.subtype
-                if subtype == localpar_code then
-                    pardir = current.dir
-                elseif subtype == dir_code then
-                    txtdir = current.dir
-                end
-            elseif id == glue_code and current.subtype == leftskip_code then -- any glue at the left?
-                --
-            elseif id == hlist_code or id == vlist_code then
--- somehow reference is true so teh following fails (second one not done) in
---    test \goto{test}[page(2)] test \gotobox{test}[page(2)]
--- so let's wait till this fails again
--- if not reference and r and (not skip or r > skip) then -- > or ~=
+            if id == hlist_code or id == vlist_code then
+                local r = current[attribute]
+                -- somehow reference is true so the following fails (second one not done) in
+                --    test \goto{test}[page(2)] test \gotobox{test}[page(2)]
+                -- so let's wait till this fails again
+                -- if not reference and r and (not skip or r > skip) then -- > or ~=
                 if r and (not skip or r > skip) then -- > or ~=
                     inject_list(id,current,r,make,stack,pardir,txtdir)
                 end
@@ -238,19 +231,31 @@ local function inject_areas(head,attribute,make,stack,done,skip,parent,pardir,tx
                 if r then
                     done[r] = done[r] - 1
                 end
-            elseif not r then
-                -- just go on, can be kerns
-            elseif not reference then
-                reference, first, last, firstdir = r, current, current, txtdir
-            elseif r == reference then
-                last = current
-            elseif (done[reference] or 0) == 0 then -- or id == glue_code and current.subtype == right_skip_code
-                if not skip or r > skip then -- maybe no > test
-                    head, current = inject_range(head,first,last,reference,make,stack,parent,pardir,firstdir)
-                    reference, first, last, firstdir = nil, nil, nil, nil
+            elseif id == whatsit_code then
+                local subtype = current.subtype
+                if subtype == localpar_code then
+                    pardir = current.dir
+                elseif subtype == dir_code then
+                    txtdir = current.dir
                 end
+            elseif id == glue_code and current.subtype == leftskip_code then -- any glue at the left?
+                --
             else
-                reference, first, last, firstdir = r, current, current, txtdir
+                local r = current[attribute]
+                if not r then
+                    -- just go on, can be kerns
+                elseif not reference then
+                    reference, first, last, firstdir = r, current, current, txtdir
+                elseif r == reference then
+                    last = current
+                elseif (done[reference] or 0) == 0 then -- or id == glue_code and current.subtype == right_skip_code
+                    if not skip or r > skip then -- maybe no > test
+                        head, current = inject_range(head,first,last,reference,make,stack,parent,pardir,firstdir)
+                        reference, first, last, firstdir = nil, nil, nil, nil
+                    end
+                else
+                    reference, first, last, firstdir = r, current, current, txtdir
+                end
             end
             current = current.next
         end
@@ -268,23 +273,29 @@ local function inject_area(head,attribute,make,stack,done,parent,pardir,txtdir)
         local current = head
         while current do
             local id = current.id
-            local r = has_attribute(current,attribute)
-            if id == whatsit_code then
+            if id == hlist_code or id == vlist_code then
+                local r = current[attribute]
+                if r and not done[r] then
+                    done[r] = true
+                    inject_list(id,current,r,make,stack,pardir,txtdir)
+                end
+                local list = current.list
+                if list then
+                    current.list = inject_area(list,attribute,make,stack,done,current,pardir,txtdir)
+                end
+            elseif id == whatsit_code then
                 local subtype = current.subtype
                 if subtype == localpar_code then
                     pardir = current.dir
                 elseif subtype == dir_code then
                     txtdir = current.dir
                 end
-            elseif id == hlist_code or id == vlist_code then
+            else
+                local r = current[attribute]
                 if r and not done[r] then
                     done[r] = true
-                    inject_list(id,current,r,make,stack,pardir,txtdir)
+                    head, current = inject_range(head,current,current,r,make,stack,parent,pardir,txtdir)
                 end
-                current.list = inject_area(current.list,attribute,make,stack,done,current,pardir,txtdir)
-            elseif r and not done[r] then
-                done[r] = true
-                head, current = inject_range(head,current,current,r,make,stack,parent,pardir,txtdir)
             end
             current = current.next
         end
@@ -294,7 +305,6 @@ end
 
 -- tracing
 
-
 local nodepool       = nodes.pool
 
 local new_rule       = nodepool.rule
@@ -303,14 +313,14 @@ local new_kern       = nodepool.kern
 local set_attribute  = node.set_attribute
 local register_color = colors.register
 
-local a_colormodel   = attributes.private('colormodel')
 local a_color        = attributes.private('color')
+local a_colormodel   = attributes.private('colormodel')
 local a_transparency = attributes.private('transparency')
 local u_transparency = nil
 local u_colors       = { }
 local force_gray     = true
 
-local function colorize(width,height,depth,n)
+local function colorize(width,height,depth,n,reference,what)
     if force_gray then n = 0 end
     u_transparency = u_transparency or transparencies.register(nil,2,.65)
     local ucolor = u_colors[n]
@@ -328,18 +338,19 @@ local function colorize(width,height,depth,n)
         u_colors[n] = u_color
     end
     if width == 0 then
-        report_area("reference %s has no horizontal dimensions: width=%s, height=%s, depth=%s",reference,width,height,depth)
+        -- probably a strut as placeholder
+        report_area("%s %s has no %s dimensions, width %p, height %p, depth %p",what,reference,"horizontal",width,height,depth)
         width = 65536
     end
     if height + depth <= 0 then
-        report_area("reference %s has no vertical dimensions: width=%s, height=%s, depth=%s",reference,width,height,depth)
+        report_area("%s %s has no %s dimensions, width %p, height %p, depth %p",what,reference,"vertical",width,height,depth)
         height = 65536/2
         depth  = height
     end
     local rule = new_rule(width,height,depth)
-    set_attribute(rule,a_colormodel,1) -- gray color model
-    set_attribute(rule,a_color,u_color)
-    set_attribute(rule,a_transparency,u_transparency)
+    rule[a_colormodel] = 1 -- gray color model
+    rule[a_color] = u_color
+    rule[a_transparency] = u_transparency
     if width < 0 then
         local kern = new_kern(width)
         rule.width = -width
@@ -374,11 +385,10 @@ nodes.references = {
 
 -- todo: get rid of n (n is just a number, can be used for tracing, obsolete)
 
-local function setreference(n,h,d,r)
+local function setreference(h,d,r)
     topofstack = topofstack + 1
     -- the preroll permits us to determine samepage (but delayed also has some advantages)
     -- so some part of the backend work is already done here
---~     stack[topofstack] = { n, h, d, codeinjections.prerollreference(r) }
     stack[topofstack] = { r, h, d, codeinjections.prerollreference(r) }
  -- texattribute[attribute] = topofstack -- todo -> at tex end
     texcount.lastreferenceattribute = topofstack
@@ -393,7 +403,7 @@ local function makereference(width,height,depth,reference)
     local sr = stack[reference]
     if sr then
         if trace_references then
-            report_reference("resolving attribute %s",reference)
+            report_reference("resolving attribute %a",reference)
         end
         local resolved, ht, dp, set, n = sr[1], sr[2], sr[3], sr[4], sr[5]
         if ht then
@@ -406,7 +416,7 @@ local function makereference(width,height,depth,reference)
             local result, current
             if trace_references then
                 local step = 65536
-                result = hpack_list(colorize(width,height-step,depth-step,2,reference)) -- step subtracted so that we can see seperate links
+                result = hpack_list(colorize(width,height-step,depth-step,2,reference,"reference")) -- step subtracted so that we can see seperate links
                 result.width = 0
                 current = result
             end
@@ -421,10 +431,10 @@ local function makereference(width,height,depth,reference)
             if cleanupreferences then stack[reference] = nil end
             return result, resolved
         elseif trace_references then
-            report_reference("unable to resolve annotation %s",reference)
+            report_reference("unable to resolve annotation %a",reference)
         end
     elseif trace_references then
-        report_reference("unable to resolve attribute %s",reference)
+        report_reference("unable to resolve attribute %a",reference)
     end
 end
 
@@ -460,7 +470,7 @@ local function makedestination(width,height,depth,reference)
     local sr = stack[reference]
     if sr then
         if trace_destinations then
-            report_destination("resolving attribute %s",reference)
+            report_destination("resolving attribute %a",reference)
         end
         local resolved, ht, dp, name, view = sr[1], sr[2], sr[3], sr[4], sr[5]
         if ht then
@@ -475,7 +485,7 @@ local function makedestination(width,height,depth,reference)
                 width, height, depth = 5*step, 5*step, 0
             end
             for n=1,#name do
-                local rule = hpack_list(colorize(width,height,depth,3,reference))
+                local rule = hpack_list(colorize(width,height,depth,3,reference,"destination"))
                 rule.width = 0
                 if not result then
                     result, current = rule, rule
@@ -491,12 +501,12 @@ local function makedestination(width,height,depth,reference)
         for n=1,#name do
             local annot = nodeinjections.destination(width,height,depth,name[n],view)
             if not result then
-                result, current = annot, annot
+                result  = annot
             else
                 current.next = annot
                 annot.prev = current
-                current = annot
             end
+            current = find_node_tail(annot)
         end
         if result then
             -- some internal error
@@ -506,7 +516,7 @@ local function makedestination(width,height,depth,reference)
         if cleanupdestinations then stack[reference] = nil end
         return result, resolved
     elseif trace_destinations then
-        report_destination("unable to resolve attribute %s",reference)
+        report_destination("unable to resolve attribute %a",reference)
     end
 end
 
@@ -531,17 +541,20 @@ function references.inject(prefix,reference,h,d,highlight,newwindow,layer) -- to
     else
         -- check
         set.highlight, set.newwindow,set.layer = highlight, newwindow, layer
-        setreference(tex.currentgrouplevel,h,d,set) -- sets attribute / todo: for set[*].error
+        setreference(h,d,set) -- sets attribute / todo: for set[*].error
     end
 end
 
 function references.injectcurrentset(h,d) -- used inside doifelse
     local currentset = references.currentset
     if currentset then
-        setreference(tex.currentgrouplevel,h,d,currentset) -- sets attribute / todo: for set[*].error
+        setreference(h,d,currentset) -- sets attribute / todo: for set[*].error
     end
 end
 
+commands.injectreference        = references.inject
+commands.injectcurrentreference = references.injectcurrentset
+
 --
 
 local function checkboth(open,close)
@@ -560,7 +573,7 @@ end
 
 statistics.register("interactive elements", function()
     if nofreferences > 0 or nofdestinations > 0 then
-        return string.format("%s references, %s destinations",nofreferences,nofdestinations)
+        return format("%s references, %s destinations",nofreferences,nofdestinations)
     else
         return nil
     end
