@@ -6,13 +6,14 @@ if not modules then modules = { } end modules ['node-typ'] = {
     license   = "see context related readme files"
 }
 
-local utfvalues      = string.utfvalues
+local utfvalues      = utf.values
 
 local currentfont    = font.current
 local fontparameters = fonts.hashes.parameters
 
 local hpack          = node.hpack
 local vpack          = node.vpack
+local fast_hpack     = nodes.fasthpack
 
 local nodepool       = nodes.pool
 
@@ -45,7 +46,9 @@ local function tonodes(str,fontid,spacing) -- quick and dirty
             next = newglyph(fontid or 1,c)
             spacedone = false
         end
-        if not head then
+        if not next then
+            -- nothing
+        elseif not head then
             head = next
         else
             prev.next = next
@@ -62,6 +65,10 @@ function typesetters.hpack(str,fontid,spacing)
     return hpack(tonodes(str,fontid,spacing),"exactly")
 end
 
+function typesetters.fast_hpack(str,fontid,spacing)
+    return fast_hpack(tonodes(str,fontid,spacing),"exactly")
+end
+
 function typesetters.vpack(str,fontid,spacing)
     -- vpack is just a hack, and a proper implentation is on the agenda
     -- as it needs more info etc than currently available
