@@ -14,7 +14,9 @@ local texsetdimen, texsetcount, texbox = tex.setdimen, tex.setcount, tex.box
 local hpack, free, copy, traverse_id = node.hpack, node.free, node.copy_list, node.traverse_id
 local texdimen, texcount = tex.dimen, tex.count
 
--- fastrepack
+local hlist_code      = nodes.nodecodes.hlist
+local box_code        = nodes.listcodes.box
+local node_dimensions = node.dimensions
 
 function commands.doreshapeframedbox(n)
     local box            = texbox[n]
@@ -28,7 +30,7 @@ function commands.doreshapeframedbox(n)
     if box.width ~= 0 then
         local list = box.list
         if list then
-            for h in traverse_id('hlist',list) do -- no dir etc needed
+            for h in traverse_id(hlist_code,list) do -- no dir etc needed
                 if not firstheight then
                     firstheight = h.height
                 end
@@ -36,8 +38,11 @@ function commands.doreshapeframedbox(n)
                 noflines = noflines + 1
                 local l = h.list
                 if l then
-                    local p = hpack(copy(l))
-                    lastlinelength = p.width
+                    if h.subtype == box_code then -- maybe more
+                        lastlinelength = h.width
+                    else
+                        lastlinelength = node_dimensions(l) -- used to be: hpack(copy(l)).width
+                    end
                     if lastlinelength > maxwidth then
                         maxwidth = lastlinelength
                     end
@@ -45,19 +50,25 @@ function commands.doreshapeframedbox(n)
                         minwidth = lastlinelength
                     end
                     totalwidth = totalwidth + lastlinelength
-                    free(p)
                 end
             end
             if firstheight then
                 if maxwidth ~= 0 then
-                    for h in traverse_id('hlist',list) do
+                    for h in traverse_id(hlist_code,list) do
                         local l = h.list
                         if l then
-                    --  if h.width ~= maxwidth then -- else no display math handling (uses shift)
-                            h.list = hpack(l,maxwidth,'exactly',h.dir)
-                            h.shift = 0 -- needed for display math
-                            h.width = maxwidth
-                    --  end
+                            if h.subtype == box_code then
+                                -- explicit box, no 'line'
+                            else
+                             -- if h.width ~= maxwidth then -- else no display math handling (uses shift)
+                                -- challenge: adapt glue_set
+                                -- h.glue_set = h.glue_set * h.width/maxwidth -- interesting ... doesn't matter much
+                                -- h.width = maxwidth
+                                    h.list = hpack(l,maxwidth,'exactly',h.dir)
+                                    h.shift = 0 -- needed for display math
+                                    h.width = maxwidth
+                             -- end
+                            end
                         end
                     end
                 end
@@ -82,7 +93,7 @@ function commands.doanalyzeframedbox(n)
     if box.width ~= 0 then
         local list = box.list
         if list then
-            for h in traverse_id('hlist',list) do
+            for h in traverse_id(hlist_code,list) do
                 if not firstheight then
                     firstheight = h.height
                 end
