@@ -3,12 +3,17 @@ if not modules then modules = { } end modules ['sort-lan'] = {
     comment   = "companion to sort-lan.mkiv",
     author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
     copyright = "PRAGMA ADE / ConTeXt Development Team",
-    license   = "see context related readme files"
+    license   = "see context related readme files",
+    dataonly  = true,
 }
 
+-- todo: look into uts#10 (2012) ... some experiments ... something
+-- to finish in winter.
+
 -- Many vectors were supplied by Wolfgang Schuster and Philipp
 -- Gesang. However this is a quite adapted and reformatted variant
--- so it needs some checking.
+-- so it needs some checking. Other users provides tables and
+-- corrections as well.
 
 local utfchar, utfbyte  = utf.char, utf.byte
 local sorters           = sorters
@@ -85,8 +90,20 @@ definitions['Duden'] = {
     replacements = { { "ß", 's' } },
 }
 
-definitions['de']    = { parent = 'default' } -- new german
+-- definitions['de'] = { parent = 'default' } -- new german
+
+definitions['de']    = {
+    parent = 'default',
+    replacements = {
+        { "ä", 'ae' }, { "Ä", 'Ae' },
+        { "ö", 'oe' }, { "Ö", 'Oe' },
+        { "ü", 'ue' }, { "Ü", 'Ue' },
+        { "ß", 's'  },
+    },
+}
+
 definitions['deo']   = { parent = 'de'      } -- old german
+
 definitions['de-DE'] = { parent = 'de'      } -- german - Germany
 definitions['de-CH'] = { parent = 'de'      } -- german - Swiss
 
@@ -863,8 +880,7 @@ definitions["et"] = {
 local fschars = characters.fschars
 
 local function firstofsplit(first)
-    local fs = fschars[first] or first
- -- print(string.format("%04x %04x",utfbyte(first),utfbyte(fs)))
+    local fs = fschars[first] or first -- leadconsonant
     return fs, fs -- entry, tag
 end
 
@@ -876,3 +892,34 @@ definitions["kr"] = {
         "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z",
     }
 }
+
+-- Japanese
+
+definitions["jp"] = {
+    replacements = {
+        { "ぁ", "あ" }, { "ぃ", "い" },
+        { "ぅ", "う" }, { "ぇ", "え" },
+        { "ぉ", "お" }, { "っ", "つ" },
+        { "ゃ", "や" }, { "ゅ", "ゆ" },
+        { "ょ", "よ" },
+    },
+    entries = {
+        ["あ"] = "あ", ["い"] = "い", ["う"] = "う", ["え"] = "え", ["お"] = "お",
+        ["か"] = "か", ["き"] = "き", ["く"] = "く", ["け"] = "け", ["こ"] = "こ",
+        ["さ"] = "さ", ["し"] = "し", ["す"] = "す", ["せ"] = "せ", ["そ"] = "そ",
+        ["た"] = "た", ["ち"] = "ち", ["つ"] = "つ", ["て"] = "て", ["と"] = "と",
+        ["な"] = "な", ["に"] = "に", ["ぬ"] = "ぬ", ["ね"] = "ね", ["の"] = "の",
+        ["は"] = "は", ["ひ"] = "ひ", ["ふ"] = "ふ", ["へ"] = "へ", ["ほ"] = "ほ",
+        ["ま"] = "ま", ["み"] = "み", ["む"] = "む", ["め"] = "め", ["も"] = "も",
+        ["や"] = "や", ["ゆ"] = "ゆ", ["よ"] = "よ",
+        ["ら"] = "ら", ["り"] = "り", ["る"] = "る", ["れ"] = "れ", ["ろ"] = "ろ",
+        ["わ"] = "わ", ["ゐ"] = "ゐ", ["ゑ"] = "ゑ", ["を"] = "を", ["ん"] = "ん",
+    },
+    orders = {
+        "あ", "い", "う", "え", "お", "か", "き", "く", "け", "こ",
+        "さ", "し", "す", "せ", "そ", "た", "ち", "つ", "て", "と",
+        "な", "に", "ぬ", "ね", "の", "は", "ひ", "ふ", "へ", "ほ",
+        "ま", "み", "む", "め", "も", "や", "ゆ", "よ",
+        "ら", "り", "る", "れ", "ろ", "わ", "ゐ", "ゑ", "を", "ん",
+    }
+}
