@@ -20,14 +20,16 @@ local freezegroup     = tasks.freezegroup
 local freezecallbacks = callbacks.freeze
 
 appendaction("processors",   "normalizers", "typesetters.characters.handler")                    -- always on
-appendaction("processors",   "normalizers", "fonts.collections.process")                         -- todo
+appendaction("processors",   "normalizers", "fonts.collections.process")                         -- disabled
 appendaction("processors",   "normalizers", "fonts.checkers.missing")                            -- disabled
 
+appendaction("processors",   "characters",  "scripts.autofontfeature.handler")
+appendaction("processors",   "characters",  "scripts.splitters.handler")                         -- disabled
 appendaction("processors",   "characters",  "typesetters.cleaners.handler")                      -- disabled
 appendaction("processors",   "characters",  "typesetters.directions.handler")                    -- disabled
 appendaction("processors",   "characters",  "typesetters.cases.handler")                         -- disabled
 appendaction("processors",   "characters",  "typesetters.breakpoints.handler")                   -- disabled
-appendaction("processors",   "characters",  "scripts.preprocess")
+appendaction("processors",   "characters",  "scripts.injectors.handler")                         -- disabled
 
 appendaction("processors",   "words",       "builders.kernel.hyphenation")                       -- always on
 appendaction("processors",   "words",       "languages.words.check")                             -- disabled
@@ -36,17 +38,15 @@ appendaction("processors",   "fonts",       "builders.paragraphs.solutions.split
 appendaction("processors",   "fonts",       "nodes.handlers.characters")                         -- maybe todo
 appendaction("processors",   "fonts",       "nodes.injections.handler")                          -- maybe todo
 appendaction("processors",   "fonts",       "nodes.handlers.protectglyphs", nil, "nohead")       -- maybe todo
-appendaction("processors",   "fonts",       "builders.kernel.ligaturing")                        -- always on
-appendaction("processors",   "fonts",       "builders.kernel.kerning")                           -- always on
+appendaction("processors",   "fonts",       "builders.kernel.ligaturing")                        -- always on (could be selective: if only node mode)
+appendaction("processors",   "fonts",       "builders.kernel.kerning")                           -- always on (could be selective: if only node mode)
 appendaction("processors",   "fonts",       "nodes.handlers.stripping")                          -- disabled (might move)
+------------("processors",   "fonts",       "typesetters.italics.handler")                       -- disabled (after otf/kern handling)
 
 appendaction("processors",   "lists",       "typesetters.spacings.handler")                      -- disabled
 appendaction("processors",   "lists",       "typesetters.kerns.handler")                         -- disabled
 appendaction("processors",   "lists",       "typesetters.digits.handler")                        -- disabled (after otf handling)
 appendaction("processors",   "lists",       "typesetters.italics.handler")                       -- disabled (after otf/kern handling)
-
--- appendaction("processors",   "fonts",       "typesetters.italics.handler")                       -- disabled (after otf/kern handling)
-
 appendaction("processors",   "lists",       "typesetters.paragraphs.handler")                    -- disabled
 
 appendaction("shipouts",     "normalizers", "nodes.handlers.cleanuppage")                        -- disabled
@@ -58,9 +58,10 @@ appendaction("shipouts",     "normalizers", "nodes.shifts.handler")
 appendaction("shipouts",     "normalizers", "structures.tags.handler")                           -- disabled
 appendaction("shipouts",     "normalizers", "nodes.handlers.accessibility")                      -- disabled
 appendaction("shipouts",     "normalizers", "nodes.handlers.backgrounds")                        -- disabled
+appendaction("shipouts",     "normalizers", "nodes.handlers.alignbackgrounds")                   -- disabled
+------------("shipouts",     "normalizers", "nodes.handlers.export")                             -- disabled
 
---~ appendaction("shipouts",     "normalizers", "nodes.handlers.export")                             -- disabled
-
+appendaction("shipouts",     "finishers",   "nodes.visualizers.handler")                         -- disabled
 appendaction("shipouts",     "finishers",   "attributes.colors.handler")                         -- disabled
 appendaction("shipouts",     "finishers",   "attributes.transparencies.handler")                 -- disabled
 appendaction("shipouts",     "finishers",   "attributes.colorintents.handler")                   -- disabled
@@ -68,19 +69,23 @@ appendaction("shipouts",     "finishers",   "attributes.negatives.handler")
 appendaction("shipouts",     "finishers",   "attributes.effects.handler")                        -- disabled
 appendaction("shipouts",     "finishers",   "attributes.viewerlayers.handler")                   -- disabled
 
+--maybe integrate relocate and families
+
 appendaction("math",         "normalizers", "noads.handlers.unscript", nil, "nohead")            -- always on (maybe disabled)
 appendaction("math",         "normalizers", "noads.handlers.variants", nil, "nohead")            -- always on
-appendaction("math",         "normalizers", "noads.handlers.families", nil, "nohead")            -- always on
 appendaction("math",         "normalizers", "noads.handlers.relocate", nil, "nohead")            -- always on
+appendaction("math",         "normalizers", "noads.handlers.families", nil, "nohead")            -- always on
+
 appendaction("math",         "normalizers", "noads.handlers.render",   nil, "nohead")            -- always on
 appendaction("math",         "normalizers", "noads.handlers.collapse", nil, "nohead")            -- always on
 appendaction("math",         "normalizers", "noads.handlers.resize",   nil, "nohead")            -- always on
-appendaction("math",         "normalizers", "noads.handlers.respace",  nil, "nohead")            -- always on
+------------("math",         "normalizers", "noads.handlers.respace",  nil, "nohead")            -- always on
 appendaction("math",         "normalizers", "noads.handlers.check",    nil, "nohead")            -- always on
 appendaction("math",         "normalizers", "noads.handlers.tags",     nil, "nohead")            -- disabled
 appendaction("math",         "normalizers", "noads.handlers.italics",  nil, "nohead")            -- disabled
 
 appendaction("math",         "builders",    "builders.kernel.mlist_to_hlist")                    -- always on
+------------("math",         "builders",    "noads.handlers.italics",  nil, "nohead")            -- disabled
 
 -- quite experimental (nodes.handlers.graphicvadjust might go away)
 
@@ -96,8 +101,17 @@ appendaction("mvlbuilders",  "normalizers", "builders.vspacing.pagehandler")
 
 appendaction("vboxbuilders", "normalizers", "builders.vspacing.vboxhandler")                     --
 
+-- experimental too
+
+appendaction("mvlbuilders","normalizers","typesetters.checkers.handler")
+appendaction("vboxbuilders","normalizers","typesetters.checkers.handler")
+
 -- speedup: only kick in when used
 
+disableaction("processors",  "scripts.autofontfeature.handler")
+disableaction("processors",  "scripts.splitters.handler")
+disableaction("processors",  "scripts.injectors.handler") -- was enabled
+disableaction("processors",  "fonts.collections.process")
 disableaction("processors",  "fonts.checkers.missing")
 disableaction("processors",  "chars.handle_breakpoints")
 disableaction("processors",  "typesetters.cleaners.handler")
@@ -122,8 +136,10 @@ disableaction("shipouts",    "attributes.effects.handler")
 disableaction("shipouts",    "attributes.negatives.handler")
 disableaction("shipouts",    "attributes.viewerlayers.handler")
 disableaction("shipouts",    "structures.tags.handler")
+disableaction("shipouts",    "nodes.visualizers.handler")
 disableaction("shipouts",    "nodes.handlers.accessibility")
 disableaction("shipouts",    "nodes.handlers.backgrounds")
+disableaction("shipouts",    "nodes.handlers.alignbackgrounds")
 disableaction("shipouts",    "nodes.handlers.cleanuppage")
 
 disableaction("shipouts",    "nodes.references.handler")
@@ -143,6 +159,9 @@ disableaction("finalizers",  "builders.paragraphs.tag")
 disableaction("math",        "noads.handlers.tags")
 disableaction("math",        "noads.handlers.italics")
 
+disableaction("mvlbuilders", "typesetters.checkers.handler")
+disableaction("vboxbuilders","typesetters.checkers.handler")
+
 freezecallbacks("find_.*_file", "find file using resolver")
 freezecallbacks("read_.*_file", "read file at once")
 freezecallbacks("open_.*_file", "open file for reading")
@@ -165,8 +184,8 @@ freezegroup("shipouts",     "finishers")
 freezegroup("mvlbuilders",  "normalizers")
 freezegroup("vboxbuilders", "normalizers")
 
---~ freezegroup("parbuilders",  "lists")
---~ freezegroup("pagebuilders", "lists")
+-----------("parbuilders",  "lists")
+-----------("pagebuilders", "lists")
 
 freezegroup("math",         "normalizers")
 freezegroup("math",         "builders")
